use chrono::{DateTime, Local};
use crate::common;
use crate::bar_modules;

#[derive(Clone)]
pub struct UnibarModuleTime {
    opts:       common::AppOptions,
    date_time:  DateTime<Local>,
}

impl UnibarModuleTime {
    // --------------------
    pub fn new(o :common::AppOptions) -> Self {
        UnibarModuleTime {
            opts:       o,
            date_time:  Local::now(),
        }
    }
}

impl bar_modules::BarModuleActions for UnibarModuleTime {

    // --------------------
    fn clear(&mut self) {
        self.date_time = Local::now();
    }

    // --------------------
    fn generate_data(&mut self) {
        if self.opts.debug {
            println!("-----> Time dump {:#?}", self.date_time);
        }
    }

    // --------------------
    fn get_content(&self) -> String {
        return format!("{}", self.date_time.format("%I:%M%p %Z"));
    }

    // --------------------
    fn get_icon(&self) -> String {
        return "🕑".to_string();
    }
}

impl bar_modules::BarModuleDebug for UnibarModuleTime {

    // --------------------
    fn post_debug(&self) {
    }
}
